#ifndef _CTRACKMAP_H_
#define _CTRACKMAP_H_

#include "global.h"
#include "cpane.h"
#include <wx/pen.h>
#include <wx/menu.h>

#define MAX_SECTORS 50

class cLap;

//-----------------------------------------------------------------------------
// The track map

class cTrackMap : public cPane
{
private:
  static wxPen s_Pen_Sel;         // pen for drawing the selected part of the track
  static wxPen s_Pen_UnSel;       // pen for drawing the non-selected part of the track
  static wxPen s_ButtonPen;       // pen for drawing the buttons
  static wxPen s_SfPen;           // pen for drawing the start/finish line
  static wxPen s_SectorPen;       // pen for drawing the sector lines
  static wxPen s_SectorDragPen;   // pen for drawing the sector line that is being dragged

  float m_Scale;                  // scaling factor for drawing
  wxCoord m_MarginX;              // X margin for drawing
  wxCoord m_MarginY;              // Y margin for drawing
  float m_MinX, m_MaxY;           // track coordinates corresponding with topleft corner of trackmap

  bool m_ShowSectorToolbar;       // show the sector buttons?
  wxCoord m_ButtonSize;           // size of sector buttons
  wxPoint m_ButtonStart;          // topleft corner of the row of buttons
  int m_SelectedButton;           // button number that is shown in 'selected' state (-1 = none)

  bool m_ShowDistance;            // show distance-in-lap values of cursor?
  wxCoord m_ValueStartY;          // Y coordinate from where the distance value is displayed

  bool m_DragMode;                // are we in drag-selecting mode?
  float m_SelStart;               // distance-in-lap where selected part of track starts
  float m_SelEnd;                 // distance-in-lap where selected part of track end
  float m_ClickPos;               // distance-in-lap of point of last mouse click
  int m_ClickMarker;              // index of sector marker of last mouse click (-1 = none)

  wxString m_TrackCode;           // code of the currently shown track

  wxMenu m_Context;               // context menu
  wxMenu* m_ResetSectorsMenu;     // "reset sectors" menu

  wxCoord Track2CoordX(float tc);
  wxCoord Track2CoordY(float tc);
  void AdjustDimensions();
  void DoPaint(wxDC& dc);
  void DrawSectorMarker(wxDC& dc, float dist);
  float DecodeMouse(wxMouseEvent& event, int* button = NULL);

  // handling sectors
  bool m_LockSectors;             // lock sectors for editing?
  bool m_SectorsChanged;          // flag to enable undoing changes
  float m_SectorEnd[MAX_SECTORS]; // distance-in-lap of end of each sector
  int m_SectorCount;              // number of slots in m_SectorEnd that are used

  wxCoord Button2CoordY(int button) const;
  int Coord2ButtonY(wxCoord coord) const;
  inline int GetButtonCount() const { return m_SectorCount + 1; }
  void SelectButton(int button);
  void DrawSectorButton(wxDC& dc, int button, bool highlight = false);

  void ClearSectors();
  bool AddSector(float dist);
  void DeleteSector(int index);
  void SetTimelineSectors();
  void SetAutodetectSectors();
  void SetSelectionSector();
  int Dist2Sector(float dist);
  int Dist2Marker(float dist);

  wxArrayString m_SectorSets;     // the saved sectors (one string for each track)
  bool GetSectorSet();
  void SaveSectorSet();

  // handling the track cursor
  bool m_CursorDrawn;             // is there currently a track cursor?
  float m_CursorDist;             // distance-in-lap of the cursor
  wxCoord m_CursorX;              // the X coordinate where the track cursor has been drawn
  wxCoord m_CursorY;

  void DrawTrackCursor(wxDC& dc, wxCoord x, wxCoord y);
  void PutTrackCursor(wxDC& dc, float distance);
  void RemoveTrackCursor(wxDC& dc);

  void ResetZoom();

  // event handlers
  void OnMouseLeftClick(wxMouseEvent& event);
  void OnMouseLeftRelease(wxMouseEvent& event);
  void OnMouseLeftDoubleClick(wxMouseEvent& event);
  void OnMouseMiddleClick(wxMouseEvent& WXUNUSED(event)) { ResetZoom(); }
  void OnMouseMove(wxMouseEvent& event);
  void OnMouseRightClick(wxMouseEvent& event);
  void OnMouseWheel(wxMouseEvent& event);
  void OnMouseEntering(wxMouseEvent& event);
  void OnMouseLeaving(wxMouseEvent& event);
  void OnMenuClick(wxCommandEvent& event);
  void OnPaint(wxPaintEvent& event);
  void OnSize(wxSizeEvent& event);

  DECLARE_EVENT_TABLE()

public:
  cTrackMap(wxWindow* parent);
  virtual ~cTrackMap();

  virtual wxSize DoGetBestSize() const { return wxSize(200, 200); }

  virtual void UpdateAll();
  virtual void DoSetTrackCursorPos(cGraphView* view, float distance, int type);
  virtual void DoSetTrackSelection(float start, float end);

  virtual void TranslateTexts();

  virtual void LoadConfig(wxRegConfig* config, const wxString& key);
  virtual void SaveConfig(wxRegConfig* config, const wxString& key);
};

#endif
